@@ -247,7 +247,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	document_signals[LOADED] =
    		g_signal_new ("loaded",
 			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
+			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GeditDocumentClass, loaded),
 			      NULL, NULL,
 			      gedit_marshal_VOID__POINTER,
@@ -582,7 +582,17 @@ gedit_document_real_loaded (GeditDocument *document, const GError *error)
 
 		g_free (data);
 	}
+	else
 	*/
+	{
+		GtkTextIter iter;
+
+		gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (document),
+						  &iter,
+						  0);
+
+		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (document), &iter);
+	}
 
 	data = gedit_metadata_manager_get (document->priv->uri,
 					   "last_searched_text");
@@ -883,25 +893,13 @@ update_document_contents (GeditDocument        *doc,
 	else
 		encoding = gedit_encoding_get_utf8 ();
 
-	if (gedit_utils_is_uri_read_only (uri))
-	{
-		gedit_debug (DEBUG_DOCUMENT, "READ-ONLY");
+	gedit_document_set_uri (doc, uri);
+	gedit_document_set_encoding (doc, encoding);
 
-		gedit_document_set_readonly (doc, TRUE);
-	}
-	else
-	{
-		gedit_debug (DEBUG_DOCUMENT, "NOT READ-ONLY");
-
-		gedit_document_set_readonly (doc, FALSE);
-	}
+	gedit_document_set_readonly (doc, gedit_utils_is_uri_read_only (uri));
 
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);
 
-	gedit_document_set_uri (doc, uri);
-
-	gedit_document_set_encoding (doc, encoding);
-
 	g_get_current_time (&doc->priv->time_of_last_save_or_load);
 	
 	return TRUE;
@@ -1748,9 +1746,10 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 	mode_t saved_umask;
 	struct stat st;
 	gchar *chars = NULL;
-	gint chars_len;
 	gint fd;
 	gint retval;
+	gsize chars_len = 0;
+	gsize new_len = 0;
 	gboolean res;
 	gboolean add_cr;
 	GtkTextIter start_iter;
@@ -1875,14 +1874,24 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 	chars = gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc),
 					   &start_iter, &end_iter, TRUE);
 
+	chars_len = strlen (chars);
+
+	add_cr = FALSE;
+	
+	if (chars_len >= 1)
+	{
+		add_cr = (*(chars + chars_len - 1) != '\n');
+	}
+	
 	if (encoding != gedit_encoding_get_utf8 ())
 	{
 		GError *conv_error = NULL;
 		gchar* converted_file_contents = NULL;
 
 		converted_file_contents = gedit_convert_from_utf8 (chars, 
-								   -1, 
+								   chars_len, 
 								   encoding,
+								   &new_len,
 								   &conv_error);
 
 		if (conv_error != NULL)
@@ -1900,17 +1909,42 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 			chars = converted_file_contents;
 		}
 	}
-
-	chars_len = strlen (chars);
-
-	add_cr = (*(chars + chars_len - 1) != '\n');
+	else
+	{
+		new_len = chars_len;
+	}
 
 	/* Save the file content */
-	res = (write (fd, chars, chars_len) == chars_len);
+	res = (write (fd, chars, new_len) == new_len);
 
 	if (res && add_cr)
+	{
 		/* Add \n if needed */
-		res = (write (fd, "\n", 1) == 1);
+		if (encoding != gedit_encoding_get_utf8 ())
+		{
+			gchar* converted_n = NULL;
+		
+			converted_n = gedit_convert_from_utf8 ("\n", 
+							       -1, 
+							       encoding,
+							       &new_len,
+							       NULL);
+			
+			if (converted_n == NULL)
+			{
+				g_warning ("Cannot add '\\n' at the end of the file.");
+			}
+			else
+			{
+				res = (write (fd, converted_n, new_len) == new_len);
+				g_free (converted_n);
+			}
+		}
+		else
+		{	
+			res = (write (fd, "\n", 1) == 1);
+		}
+	}
 
 	if (!res)
 	{
@@ -2216,6 +2250,7 @@ gedit_document_end_user_action (GeditDocument *doc)
 	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (doc));
 }
 
+/* FIXME: line should be an "int" -- Paolo (Feb. 20, 2005) */
 void
 gedit_document_goto_line (GeditDocument* doc, guint line)
 {
